/*
    Copyright (C) 2004, Wolfgang Hottgenroth

    This file is part of smmapdfw.

    smmapdfw is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  

    smmapdfw is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details. 

    You should have received a copy of the GNU General Public License
    along with smmapdfw. If not, write to the Free Software Foundation, 59
    Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <syslog.h>
#include <sys/select.h>
#include <errno.h>

#include "safe_write.h"


ssize_t safe_write(int fd, const void *buf, size_t count) {
  int res;
  fd_set rdfs, wrfs;
  struct timeval tv = {0, 0};
  char read_buf[_SAFE_WRITE_READ_BUF_SIZE+1];

  FD_ZERO(&rdfs);
  FD_ZERO(&wrfs);
  FD_SET(fd, &rdfs);
  FD_SET(fd, &wrfs);
  
  while (1) {
    res = select(fd+1, &rdfs, &wrfs, NULL, &tv);
    if (-1 == res) {
      syslog(LOG_DEBUG, "_safe_write: error in select: %d, %s", errno, strerror(errno));
      return -1;
    } else {
      if (FD_ISSET(fd, &rdfs)) {
	/* set to read: something is wrong */
	syslog(LOG_DEBUG, "_safe_write, set to read");
	res = read(fd, read_buf, _SAFE_WRITE_READ_BUF_SIZE);
	syslog(LOG_DEBUG, "_safe_write, read res: %d", res);
	if (0 == res) {
	  syslog(LOG_DEBUG, "_safe_write, read returns 0, socket probably closed");
	  return -1;
	}
      } else if (FD_ISSET(fd, &wrfs)) {
	/* set to write */
	syslog(LOG_DEBUG, "_safe_write, set to write");
	res = write(fd, buf, count);
	return res;
      } else {
	syslog(LOG_DEBUG, "_safe_write: whether read nor write ready, strange");
	return -1;
      }
    }
  }
}

